/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zauberlabs.commons.mom;

import net.sf.staccatocommons.restrictions.check.NotEmpty;

import org.apache.commons.beanutils.ConvertUtilsBean;

import com.zauberlabs.commons.mom.converter.ConvertUtilsTypeConverter;
import com.zauberlabs.commons.mom.converter.impl.DateToXMLGregorianCalendarTypeConverter;
import com.zauberlabs.commons.mom.converter.impl.StringToEnumTypeConverter;
import com.zauberlabs.commons.mom.internal.DefaultMapObjectMapperBuilder;
import com.zauberlabs.commons.mom.style.impl.BeanStyle;


/**
 * {@link MapObjectMappers} is a hub for creating common
 * {@link MapObjectMapper}s and {@link MapObjectMapperBuilder}s
 *
 * @author flbulgarelli
 */
public final class MapObjectMappers {

    /**
     * Hub class
     */
    private MapObjectMappers() {
    }

    private static final ConvertUtilsTypeConverter BEAN_UTILS_CONVERTER = new ConvertUtilsTypeConverter(
            new ConvertUtilsBean());

    /**
     * Answers a basic {@link MapObjectMapper}, which has bean style as
     * setter, naming and typing styles, with no converters nor interceptor
     *
     * @param packageName the name of the mapping package
     * @return a new basic {@link MapObjectMapper} for the given pacakgeName
     */
    public static MapObjectMapper basic(@NotEmpty final String packageName) {
        return new DefaultMapObjectMapperBuilder(packageName).build();
    }

    /**
     * Answers a basic {@link MapObjectMapperBuilder}, which has {@link BeanStyle} as
     * setterSyle, with no converters no interceptor
     *
     * @param packageName the name of the mapping package
     * @return a new basic {@link MapObjectMapperBuilder} for the given pacakgeName
     */
    public static MapObjectMapperBuilder withPackage(@NotEmpty final String packageName) {
        return new DefaultMapObjectMapperBuilder(packageName);
    }

    /**
     * Answers a default {@link MapObjectMapperBuilder}, which has {@link BeanStyle} as
     * setterSyle, and the following converters in the given order:
     * <ol>
     * <li>A {@link DateToXMLGregorianCalendarTypeConverter}</li>
     * <li>A {@link StringToEnumTypeConverter}</li>
     * <li>A {@link ConvertUtilsTypeConverter}</li>
     * </ol>
     *
     * @param packageName the name of the mapping package
     * @return a new default {@link MapObjectMapperBuilder} for the given pacakgeName
     */
    public static MapObjectMapperBuilder defaultWithPackage(@NotEmpty final String packageName) {
        return withPackage(packageName) //
                .withConverter(DateToXMLGregorianCalendarTypeConverter.converter()) //
                .withConverter(StringToEnumTypeConverter.converter()) //
                .withConverter(BEAN_UTILS_CONVERTER);
    }

    /**
     * Answers a defaut {@link MapObjectMapper}, which has {@link BeanStyle} as
     * setterSyle, and the following converters in the given order:
     * <ol>
     * <li>A {@link DateToXMLGregorianCalendarTypeConverter}</li>
     * <li>A {@link StringToEnumTypeConverter}</li>
     * <li>A {@link ConvertUtilsTypeConverter}</li>
     * </ol>
     *
     * @param packageName the name of the mapping package
     * @return a new default {@link MapObjectMapper} for the given pacakgeName
     */
    public static MapObjectMapper default_(@NotEmpty final String packageName) {
        return defaultWithPackage(packageName).build();
    }



}
